#!/bin/bash

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:~/bin
export PATH
LANG=en_US.UTF-8

# ==========================================特别说明========================================== #
# 1.以G_开头的变量约定为全局变量(G_XXX)，避免误会局部变量不要以此开头，同时用 local XXX 来声明       #
# 2.Nacos 的默认账码为 nacos @Lingyun2022，配置在yaml/mysql/sqls/nacos-mysql.sql                #
# 3.RocketMQ Dashboard 账码为 admin @Lingyun2022，配置在 apply_rocketmq_dashboard               #
# 4.其他涉及账号密码的请统一查看yaml/resources/resources.yaml                                    #
# ============================================================================================ #

while getopts ":a:n:c:" opt
do
    case $opt in
        a)
        G_PARAM_A=$OPTARG
        ;;
        n)
        G_PARAM_N=$OPTARG
        ;;
        c)
        G_PARAM_C=$OPTARG
        ;;
        ?)
        echo -e "参数无效\n\nusage: $0 -a <k3s master ip> -n <namesapce>"
        exit 1;;
    esac
done

G_OS_ARCH=$(uname -m)
G_GITHUB_PROXY=https://ghfast.top

# 获取安装包所在路径
G_OS_NAME=$(grep '^NAME=' /etc/os-release | cut -d '"' -f 2)
G_INSTALL_DIR=$(cd "$(dirname "$0")";pwd)
G_INSTALL_LOG=$G_INSTALL_DIR/install-`date +%Y%m%d%H%M`.log
G_DATA_DIR=$G_INSTALL_DIR/data
mkdir -p $G_DATA_DIR

# 默认命名空间(-n pai-cloud)
G_NAMESPACE=${G_PARAM_N:-pai-cloud}
# 8.0.25  5.7.36
G_MYSQL_VERSION=8.0.30

# K3s Agent安装时要指定IP(-a 192.168.1.20)
G_MASTER_IP=$G_PARAM_A
# K3s 集群模式标识(-c y)
IS_CLUSTER_FLAG=$G_PARAM_C

# 网络模式 1 在线 0 离线
G_ONLINE=1
# 运行时： 1 Docker 0 Containerd
G_USE_DOCKER=${USE_DOCKER:-1}

# ====================================================
let count=0

step () {
    let count+=1
    echo ''
    info "============= STEP $count: $1 ============="
}

# --- helper functions for logs ---
info () {
    echo -e "`date +%Y-%m-%dT%H:%M:%S`" ' [INFO ] ' "$@"
}

warn () {
    echo -e "`date +%Y-%m-%dT%H:%M:%S`" ' [WARN ] ' "$@" >&2
}

fatal () {
    echo -e "`date +%Y-%m-%dT%H:%M:%S`" ' [ERROR] ' "$@" >&2
    info "安装失败，退出程序"
    exit 1
}


# ====================================================
check_space () {
    # 检查根路径的剩余空间
    local available_space=$(df / | tail -1 | awk '{print $4}')

    # 换算为GB
    local available_space_gb=$((available_space / 1024 / 1024))

    read -p "当前根路径 (/) 可用空间为：${available_space_gb}GB，是否继续(Y/N)？" confirm

    # 将用户输入的字符转换为大写
    confirm=$(echo "$confirm" | tr 'a-z' 'A-Z')

    if [ "$confirm" != "Y" ]; then
        exit 1
    fi
}

if [ $G_USE_DOCKER -ne 1 ]; then
    info "正在使用Containerd作为运行时，无法改变存储路径，默认存在 / 下"
    check_space
else
    info "正在使用Docker作为运行时, 存储路径在/etc/docker/daemon.json配置"
fi

# ====================================================


download_if_not_exist () {
    local FILE=$1
    local URL=$2    
    if [[ ! -f "$FILE" ]]; then
        info "开始下载: \033[32m $FILE \033[0m "
        curl -# -o $FILE -C - $URL
        # wget -c -O $FILE $URL
    else
        info "文件 \033[32m $FILE \033[0m 已存在"
    fi
}

pull_if_not_exist () {
    local IMG_NAME=$1
    local IMG_VERSION=$2
    local EXEC_OPTION=$3

    if [[ `crictl images |grep -v grep |grep $IMG_NAME |grep $IMG_VERSION |wc -l` -eq 0 ]]; then
         if [[ $EXEC_OPTION == "backend" ]]; then
            crictl pull "$IMG_NAME:$IMG_VERSION" &
            info "后台拉取: \033[32m $IMG_NAME:$IMG_VERSION \033[0m "
        else
            info "开始拉取: \033[32m $IMG_NAME:$IMG_VERSION \033[0m "
            crictl pull "$IMG_NAME:$IMG_VERSION"
        fi
    else
        info "已存在镜像:  \033[32m $IMG_NAME:$IMG_VERSION \033[0m "
    fi
}

get_resource () {
    local MODULE="$1"
    local NAME="$2"
    info "find resource condition: $MODULE.$NAME"
    # tail -n +1000         ：从1000行开始显示，显示1000行(含)以后的内容
    # head -n 1000          ：显示前面1000行
    local POS=$(cat -n $G_INSTALL_DIR/yaml/resources/resources.yaml | grep "$MODULE:" | tail -n +1 | head -n 1 | awk '{print $1}')    
    G_RESOURCE_VALUE=$(cat $G_INSTALL_DIR/yaml/resources/resources.yaml | tail -n +"$POS" | head -n 5 | grep "$NAME" | awk '{print $NF}' | sed "s+'++g" | tr -d '\r')
    info "find resource resultset: begin row -> [$POS] , value -> [$G_RESOURCE_VALUE]"
}

#检测网络链接畅通
check_network() {

    if [ "$G_ONLINE" = "0" ]; then
        warn "注意：启用了离线模式"
        return 0
    fi

    #超时时间
    local TIMEOUT=5
    #目标网站
    local TARGET=www.baidu.com
    #获取响应状态码
    local RET_CODE=`curl -I -s --connect-timeout ${TIMEOUT} ${TARGET} -w %{http_code} | tail -n1`
    if [ "x$RET_CODE" = "x200" ]; then
        #网络畅通
        return 0
    fi
   
    read -p "网络不畅通，请确认是否已准备了离线包并转为离线模式? （Y/N）： " IS_OFFLIE           
    if [[ "$IS_OFFLIE" = [yY] ]]; then
        G_ONLINE=0
        info "网络不畅通，转为离线模式"
    else 
        G_ONLINE=0 # 也设置为0，让同步脚本不要执行
        fatal "网络不畅通，请检查网络设置！"
    fi
}
# ====================================================
system_init () {
    if [ "$G_ONLINE" = "1" ]; then            
        if [[ "$G_OS_NAME" == "CentOS Linux" ]]; then
        
          step "设置镜像源，安装wget"
          if ! grep -q aliyun.com /etc/yum.repos.d/CentOS-Base.repo; then
              cd /etc/yum.repos.d/
  
              # 先备份一下原先的yum源，避免出错无法恢复
              mv CentOS-Base.repo CentOS-Base.repo.`date +%y%m%d%H%M`.bak
              if [[ -f "epel.repo" ]]; then 
                  mv epel.repo epel.repo.`date +%y%m%d%H%M`.bak 
              fi
  
              # 修改base.reop源
              curl -o CentOS-Base.repo http://mirrors.aliyun.com/repo/Centos-7.repo
  
              # 安装epel.repo源
              curl -o /etc/yum.repos.d/epel.repo http://mirrors.aliyun.com/repo/epel-7.repo
  
              # 刷新缓存
              yum clean all
              yum makecache
          fi
  
  
          if ! [ -x "$(command -v yum-config-manager)" ]; then
             # 为安装Docker时使用yum-config-manager准备
              yum install -y yum-utils
          fi
              
          
          # 存在则不安装
          if ! [ -x "$(command -v wget)" ]; then
              yum install -y wget
          fi
        elif [[ "$G_OS_NAME" == "openEuler" ]]; then
          dnf install -y dnf-plugins-core
          # 存在则不安装
          if ! [ -x "$(command -v wget)" ]; then
              dnf install -y wget
          fi
        else
            step "未支持的系统，跳过初始化"
        fi
    fi
}

close_firewalld () {
    step "关闭防火墙"
    # 停止并禁用防火墙
    systemctl disable --now firewalld

    # 禁用selinux
    sed -i 's/^SELINUX=enforcing/SELINUX=disabled/' /etc/selinux/config
    sed -i 's/^SELINUX=permissive/SELINUX=disabled/' /etc/selinux/config
    # 禁用配置修改后未重启，先临时关闭selinux
    setenforce 0
}

close_swap () {
    step "禁止swap分区"
    sed -i 's/^[^#].*swap.*/#&/' /etc/fstab
    # 禁用配置修改后未重启，先临时关闭
    swapoff -a
}

uninstall_k3s () {
    step "检测删除旧k3s"
    if [[ -e /usr/local/bin/k3s-uninstall.sh ]]; then
        sh /usr/local/bin/k3s-uninstall.sh
    fi
    if [[ -e /usr/local/bin/k3s-agent-uninstall.sh ]]; then
        sh /usr/local/bin/k3s-agent-uninstall.sh
    fi
    [ -f /root/.kube/config ] && rm /root/.kube/config
}

uninstall_docker () {
    step "检测删除 Docker 及依赖组件 和 数据目录"
    if command -v docker >/dev/null 2>&1; then
        warn "准备卸载Docker......"
        local containers=$(docker ps -a -q)
        if [ -z "$containers" ]; then
            info "没有找到容器，跳过删除操作"
        else
            docker rm -f $containers
            info "所有容器已删除"
        fi
        systemctl stop docker 
        systemctl disable docker 
        # Uninstall old versions
        yum remove -y docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine
        # Uninstall Docker Engine
        yum remove -y docker-ce docker-ce-cli containerd.io
        umount `cat /proc/mounts | grep "docker"| awk '{print \$2}'`
        rm -rf /var/lib/docker
        rm -rf /var/run/docker
        rm -rf /etc/docker
        rm -rf /usr/libexec/docker
        rm -f /usr/bin/docker
        rm -rf /usr/share/kuboard
    fi    
}

install_docker_by_tgz () {
    local ver=26.1.4
    local tgz=docker-"$ver".tgz
    local url=https://download.docker.com/linux/static/stable/"$G_OS_ARCH"/"$tgz"

    download_if_not_exist "$G_INSTALL_DIR"/libs/"$tgz" "$url"
    tar -xvf "$G_INSTALL_DIR"/libs/"$tgz"

    cp -p "$G_INSTALL_DIR"/libs/docker/* /usr/bin

cat > /etc/systemd/system/docker.service << 'EOF'
[Unit]
Description=Docker Application Container Engine
Documentation=https://docs.docker.com
After=network-online.target firewalld.service
Wants=network-online.target

[Service]
Type=notify
# the default is not to use systemd for cgroups because the delegate issues still
# exists and systemd currently does not support the cgroup feature set required
# for containers run by docker
ExecStart=/usr/bin/dockerd
ExecReload=/bin/kill -s HUP $MAINPID
# Having non-zero Limit*s causes performance problems due to accounting overhead
# in the kernel. We recommend using cgroups to do container-local accounting.
LimitNOFILE=infinity
LimitNPROC=infinity
LimitCORE=infinity
# Uncomment TasksMax if your systemd version supports it.
# Only systemd 226 and above support this version.
#TasksMax=infinity
TimeoutStartSec=0
# set delegate yes so that systemd does not reset the cgroups of docker containers
Delegate=yes
# kill only the docker process, not all processes in the cgroup
KillMode=process
# restart the docker process if it exits prematurely
Restart=on-failure
StartLimitBurst=3
StartLimitInterval=60s

[Install]
WantedBy=multi-user.target

EOF
}

install_docker () {
    step "安装 Docker 及 依赖组件 启动服务"
    if [ "$G_ONLINE" = "1" ]; then
        if [[ "$G_OS_NAME" == "CentOS Linux" ]]; then
            # 当有的内网需要登录时会变成一段HTML，需先删除再重新下载
            if grep -q "DOCTYPE html" /etc/yum.repos.d/docker-ce.repo; then
               rm -f /etc/yum.repos.d/docker-ce.repo
            fi

            yum-config-manager --add-repo http://mirrors.aliyun.com/docker-ce/linux/centos/docker-ce.repo
            # wget -O /etc/yum.repos.d/docker-ce.repo https://repo.huaweicloud.com/docker-ce/linux/centos/docker-ce.repo
            # sed -i 's+download.docker.com+repo.huaweicloud.com/docker-ce+' /etc/yum.repos.d/docker-ce.repo
            yum makecache fast
            #yum install -y device-mapper-persistent-data lvm2 docker-ce-19.03.9 docker-ce-cli-19.03.9 containerd.io
            yum install -y device-mapper-persistent-data lvm2 docker-ce docker-ce-cli containerd.io
        elif [[ "$G_OS_NAME" == "openEuler" ]]; then
            dnf config-manager --add-repo=https://repo.huaweicloud.com/docker-ce/linux/centos/docker-ce.repo
            #sed -i 's+download.docker.com+repo.huaweicloud.com/docker-ce+' /etc/yum.repos.d/docker-ce.repo
            sed -i 's+download.docker.com+mirrors.aliyun.com/docker-ce+' /etc/yum.repos.d/docker-ce.repo
            sed -i 's+$releasever+8+'  /etc/yum.repos.d/docker-ce.repo
            dnf makecache
            #dnf install -y device-mapper-persistent-data lvm2 docker-ce docker-ce-cli containerd.io
            dnf install -y device-mapper-persistent-data lvm2 docker-ce-25.0.5 docker-ce-cli-25.0.5 containerd.io
        else
            install_docker_by_tgz
        fi
    elif [ -d "$G_INSTALL_DIR/libs/docker" ]; then
        # 离线安装
        cd $G_INSTALL_DIR/libs/docker
        rpm -Uvh --force --nodeps *.rpm
    else 
        fatal "安装 Docker 失败"
    fi
    mkdir -p /etc/docker

# 注意这个的EOF前面不能有空格，否则会报错
cat << EOF > /etc/docker/daemon.json 
{
  "registry-mirrors": [ 
    "https://xwcbybgr.mirror.aliyuncs.com", 
    "https://registry.docker-cn.com", 
    "https://hub-mirror.c.163.com", 
    "https://docker.mirrors.ustc.edu.cn" 
  ], 
  "max-concurrent-downloads": 10,
  "experimental": true, 
  "log-driver":"json-file", 
  "log-opts": {"max-size":"100m", "max-file":"3"},
  "data-root":"$G_DATA_DIR/docker"
}
EOF
# 注意这个的EOF前面不能有空格，否则会报错

    systemctl enable docker
    systemctl start docker
    systemctl status docker

}

prepare_images () {
    # 检查是否有离线包，有则先加载避免反正下载
    if [ -d "$G_INSTALL_DIR/libs/" ]; then
        info "导入离线镜像......"
        if command -v docker >/dev/null 2>&1; then
            find $G_INSTALL_DIR/libs/ -maxdepth 2 -name "*.tar" -type f -exec docker load -i {} \;
        else
            find $G_INSTALL_DIR/libs/ -maxdepth 2 -name "*.tar" -type f -exec ctr -n k8s.io images import {} \;
        fi
    fi

    sleep 10s

    # G_MASTER_IP 为空表示安装master 否则表示安装agent
    if [ -z ${G_MASTER_IP} ]; then
        # 预先拉取这两个镜像
        pull_if_not_exist  'dev.flyrise.cn:8082/library/mysql' "$G_MYSQL_VERSION"
        pull_if_not_exist  'dev.flyrise.cn:8082/library/nacos-server' '2.0.4-slim'

        # 主节点上安装且不是集群模式先拉取ingress nginx controller镜像
        if [[ "$IS_CLUSTER_FLAG" != [yY] ]]; then
            pull_if_not_exist  'dev.flyrise.cn:8082/library/nginx-ingress-controller' 'v1.0.0'
            pull_if_not_exist  'dev.flyrise.cn:8082/library/kube-webhook-certgen' 'v1.0'
        fi

        pull_if_not_exist  'dev.flyrise.cn:8082/library/xxl-job-admin' '2.2.0' 'backend'
        pull_if_not_exist  'dev.flyrise.cn:8082/library/rocketmq' '4.9.4' 'backend'
        pull_if_not_exist  'dev.flyrise.cn:8082/library/redis' '6.2.3' 'backend'
        pull_if_not_exist  'dev.flyrise.cn:8082/library/minio' 'RELEASE.2021-06-14T01-29-23Z' 'backend'
        pull_if_not_exist  'dev.flyrise.cn:8082/library/elasticsearch' '7.17.2' 'backend'
    fi
}


install_k3s () {
    step "安装 K3s 启动服务"

    # v1.17.4+k3s1
    # v1.20.14+k3s1
    # v1.21.1+k3s1
    # v1.21.8+k3s1
    # v1.22.4+k3s1
    # v1.22.5+k3s1  ingress 不能正常运行
    local VERSION=v1.21.1+k3s1
    local WORKPATH=$G_INSTALL_DIR/libs/k3s
    local ARCH=x86_64
    if [ "$G_OS_ARCH" == "x86_64" ]; then
        ARCH=amd64
    elif [ "$G_OS_ARCH" == "aarch64" ]; then
        ARCH=arm64
    fi
    mkdir -p $WORKPATH/$VERSION
    cd $WORKPATH
        

    download_if_not_exist "$WORKPATH/$VERSION/k3s" "$G_GITHUB_PROXY/https://github.com/k3s-io/k3s/releases/download/$VERSION/k3s"
    download_if_not_exist "$WORKPATH/$VERSION/k3s-airgap-images-$ARCH.tar.gz" "$G_GITHUB_PROXY/https://github.com/k3s-io/k3s/releases/download/$VERSION/k3s-airgap-images-$ARCH.tar.gz"
    download_if_not_exist "$WORKPATH/install.sh" "https://rancher-mirror.rancher.cn/k3s/k3s-install.sh"
       
    #download_if_not_exist "$WORKPATH/$VERSION/k3s" "https://ghp.ci/https://github.com/k3s-io/k3s/releases/download/$VERSION/k3s"
    #download_if_not_exist "$WORKPATH/$VERSION/k3s-airgap-images-amd64.tar.gz" "https://ghp.ci/https://github.com/k3s-io/k3s/releases/download/$VERSION/k3s-airgap-images-amd64.tar.gz"
    #download_if_not_exist "$WORKPATH/k3s-selinux-0.2-1.el7_8.noarch.rpm" "https://rpm.rancher.io/k3s/stable/common/centos/7/noarch/k3s-selinux-0.2-1.el7_8.noarch.rpm"
    #download_if_not_exist "$WORKPATH/install.sh" "https://rancher-mirror.rancher.cn/k3s/k3s-install.sh"
    #download_if_not_exist "$WORKPATH/install.sh" "https://get.k3s.io/"

    # download_if_not_exist "$WORKPATH/$VERSION/k3s" "https://api.flyrise.cn/pai/k3s/$VERSION/k3s"
    # download_if_not_exist "$WORKPATH/$VERSION/k3s-airgap-images-amd64.tar.gz" "https://api.flyrise.cn/pai/k3s/$VERSION/k3s-airgap-images-amd64.tar.gz"
    # #download_if_not_exist "$WORKPATH/k3s-selinux-0.2-1.el7_8.noarch.rpm" "https://api.flyrise.cn/pai/k3s/stable/common/centos/7/noarch/k3s-selinux-0.2-1.el7_8.noarch.rpm"
    # #download_if_not_exist "$WORKPATH/k3s-selinux-1.1-1.el7.noarch.rpm" "https://api.flyrise.cn/pai/k3s/stable/common/centos/7/noarch/k3s-selinux-1.1-1.el7.noarch.rpm"
    # download_if_not_exist "$WORKPATH/k3s-selinux-1.2-2.el7.noarch.rpm" "https://api.flyrise.cn/pai/k3s/stable/common/centos/7/noarch/k3s-selinux-1.2-2.el7.noarch.rpm"
    # #download_if_not_exist "$WORKPATH/install.sh" "https://rancher-mirror.rancher.cn/k3s/k3s-install.sh"
    # download_if_not_exist "$WORKPATH/install.sh" "https://api.flyrise.cn/pai/k3s/install.sh"
    
    if [ "${G_ONLINE}" = "1" ]; then
        yum install -y container-selinux selinux-policy-base
    fi
    rpm -Uvh --force --nodeps *.rpm
    
    chmod +x ./$VERSION/k3s && cp ./$VERSION/k3s /usr/local/bin/k3s

    mkdir -p /var/lib/rancher/k3s/agent/images/
    cp $WORKPATH/$VERSION/k3s-airgap-images-"$ARCH".tar.gz /var/lib/rancher/k3s/agent/images/

    local INSTALL_OPTS="--disable traefik --kubelet-arg max-pods=200"
    if [ $G_USE_DOCKER -eq 1 ]; then   
       INSTALL_OPTS="$INSTALL_OPTS --docker" 
       docker load -i /var/lib/rancher/k3s/agent/images/k3s-airgap-images-"$ARCH".tar.gz
    else
       gzip -df /var/lib/rancher/k3s/agent/images/k3s-airgap-images-"$ARCH".tar.gz
    fi

    info "INSTALL_OPTS: $INSTALL_OPTS"

    # G_MASTER_IP 为空表示安装master 否则表示安装agent
    if [ -z ${G_MASTER_IP} ]; then
        info "install k3s master"
        INSTALL_K3S_SKIP_DOWNLOAD=true INSTALL_K3S_EXEC="$INSTALL_OPTS" sh ./install.sh

        if [[ "$IS_CLUSTER_FLAG" = [yY] ]]; then
            echo "K3s Master已安装完毕，请到Worker节点执行以下命令："
            echo "sh cei_install.sh -a <MASTER IP>"
            read -p "是否Worker节点安装完成？ [回车继续]： " IS_AGENT_JOIN
        fi

        if [[ ! -f /root/.kube/config ]]; then
            mkdir -p /root/.kube
            cp /etc/rancher/k3s/k3s.yaml /root/.kube/config
            #local local_ip=$(ip a | grep $(route -n | grep ^0.0.0.0 | awk '{print $8}') | grep "inet " | awk '{print $2}' |awk -F '/' '{print $1;exit;}')
            local_ip=$(hostname -I | awk '{print $1}')
            sed -i "s+server: https://.*:6443$+server: https://$local_ip:6443+" /root/.kube/config
        fi
    else
        read -p "是否Master节点已提示执行安装Worker节点命令 [默认为 N] ? （Y/N）： " IS_MASTER_READY

        if [[ "$IS_MASTER_READY" = [yY] ]]; then
            local MASTER_TOKEN=$(ssh "$G_MASTER_IP" cat /var/lib/rancher/k3s/server/node-token)
            info "install k3s agent, token is : $MASTER_TOKEN"
            INSTALL_K3S_SKIP_DOWNLOAD=true K3S_URL="https://$G_MASTER_IP:6443" K3S_TOKEN="$MASTER_TOKEN" INSTALL_K3S_EXEC="$INSTALL_OPTS" sh install.sh
        fi
    fi
}


apply_ingress_controller () {
    step "安装 ingress nginx controller"
    cd $G_INSTALL_DIR/yaml/ingress-nginx

    kubectl apply -f ingress-nginx-controller.yaml
}

apply_kuboard () {
    step "安装 Kuboard(kuboard-v3、 etcd-host、 agent)"    
    cd $G_INSTALL_DIR/yaml/kuboard
    rm -f kuboard-v3-install.yaml

    local LIBSPATH=$G_INSTALL_DIR/libs/kuboard
    mkdir -p $LIBSPATH
    
    # Harbor image
    cp kuboard-v3.yaml kuboard-v3-install.yaml

    download_if_not_exist "$LIBSPATH/kuboard-v3.tar.gz" "https://api.flyrise.cn/pai/kuboard/kuboard-v3.tar.gz"

    if command -v docker >/dev/null 2>&1; then
        docker load -i $LIBSPATH/kuboard-v3.tar.gz
    else
        gzip -dc $LIBSPATH/kuboard-v3.tar.gz > $LIBSPATH/kuboard-v3.tar
        ctr -n k8s.io images import $LIBSPATH/kuboard-v3.tar
        rm -f $LIBSPATH/kuboard-v3.tar
    fi

    # 修改 Agent 与 Kuboard 通信时的密钥
    sed -i "s+32b7d6572c6255211b4eec9009e4a816+$(echo `date +%s` | md5sum |awk '{print $1}')+g" kuboard-v3-install.yaml
    kubectl apply -f kuboard-v3-install.yaml
}

change_kuboard_policy () {
    for i in {1..20}; do
        [[ `kubectl get po -nkuboard|grep agent|wc -l` -eq 2 ]] && break
        sleep 10s
    done
    # 更改镜像拉取策略
    kubectl patch deployment kuboard-agent   -n kuboard --patch '{"spec": {"template": {"spec": {"containers": [{"name": "kuboard-agent","imagePullPolicy":"IfNotPresent"}]}}}}'
    kubectl patch deployment kuboard-agent-2 -n kuboard --patch '{"spec": {"template": {"spec": {"containers": [{"name": "kuboard-agent","imagePullPolicy":"IfNotPresent"}]}}}}'
    
    
    local EXCODE=$?  
    if [ "$EXCODE" -ne "0" ]; then  
        echo ""
        echo "kubectl patch deployment kuboard-agent    -n kuboard --patch '{\"spec\": {\"template\": {\"spec\": {\"containers\": [{\"name\": \"kuboard-agent\",\"imagePullPolicy\":\"IfNotPresent\"}]}}}}'"
        echo "kubectl patch deployment kuboard-agent-2  -n kuboard --patch '{\"spec\": {\"template\": {\"spec\": {\"containers\": [{\"name\": \"kuboard-agent\",\"imagePullPolicy\":\"IfNotPresent\"}]}}}}'"
        echo ""
        warn "请手动执行 kubectl get po -n kuboard 检查 kuboard-agent 和 kuboard-agent-2 部署状态，待得出现后使用以上命令更改镜像拉取策略" 
    fi
}


create_namespace () {
    step "创建 $G_NAMESPACE 命名空间"
    if kubectl get ns|grep $G_NAMESPACE; then
        info 已存在 $G_NAMESPACE 
    else
        #kubectl create namespace $G_NAMESPACE --dry-run=client -o yaml
        kubectl create namespace $G_NAMESPACE
    fi
}


check_database () {
    local MYSQL_POD="`kubectl get po -l 'app=mysql' -n $G_NAMESPACE|grep -v NAME |awk '{print $1}'`"
    get_resource "mysql" "password"
    local MYSQL_ROOT_PASSWORD=$G_RESOURCE_VALUE
    info "\033[32m `date` \033[0m 正等待 MySQL 服务启动, 侦测到实例为: $MYSQL_POD" 
    sleep 30s

    let flag=0
    while [[ $flag -eq 0 ]]; do
        kubectl exec $MYSQL_POD -n $G_NAMESPACE -- mysql --user=root --password=$MYSQL_ROOT_PASSWORD --execute='show databases;'  
      
        if [[ $? -eq 0 ]]; then
            let flag=1
        fi
        sleep 3s
    done
}

apply_mysql () {
    step "部署 MySQL 中间件"        
    get_resource "mysql" "password"
    local MYSQL_ROOT_PASSWORD=$G_RESOURCE_VALUE    

    local MYSQL_DATA=$G_DATA_DIR/mysql/data
    local MYSQL_CONFIG=$G_DATA_DIR/mysql/config
    local INIT_SQLS=$G_INSTALL_DIR/yaml/mysql/sqls
    mkdir -p $MYSQL_DATA
    mkdir -p $MYSQL_CONFIG

    if [ -f "$INIT_SQLS/database.sql" ]; then
      if grep -q "CREATE DATABASE IF NOT EXISTS \`xxl_job\`" "$INIT_SQLS/database.sql"; then
        if [ -f "$INIT_SQLS/xxl-job.tpl" ]; then
          mv $INIT_SQLS/xxl-job.tpl $INIT_SQLS/xxl-job.sql
        fi
      fi
    fi

    cd $G_INSTALL_DIR/yaml/mysql
    rm -f mysql-install.yaml
    cp mysql.yaml mysql-install.yaml
    sed -i "s+your-mysql-version+$G_MYSQL_VERSION+g" mysql-install.yaml
    sed -i "s+your-mysql-root-password+$MYSQL_ROOT_PASSWORD+g" mysql-install.yaml
    sed -i "s+your-mysql-data-path+$MYSQL_DATA+g" mysql-install.yaml
    sed -i "s+your-mysql-config-path+$MYSQL_CONFIG+g" mysql-install.yaml
    sed -i "s+your-mysql-init-sqls-path+$INIT_SQLS+g" mysql-install.yaml


    kubectl apply -f mysql-install.yaml

    check_database
}

apply_redis () {
    step "部署 Redis 中间件"    
    get_resource "redis" "password"
    local REDIS_PASSWORD=$G_RESOURCE_VALUE  
    local REDIS_DATA=$G_DATA_DIR/redis/data
    mkdir -p $REDIS_DATA

    cd $G_INSTALL_DIR/yaml/redis
    rm -f redis-install.yaml
    cp redis.yaml redis-install.yaml
    sed -i "s+your-redis-password+$REDIS_PASSWORD+g" redis-install.yaml
    sed -i "s+your-redis-data-path+$REDIS_DATA+g" redis-install.yaml

    kubectl apply -f redis-install.yaml
}

apply_nacos () {
    step "部署 Nacos 中间件"
    get_resource "mysql" "host"
    local MYSQL_HOST=$G_RESOURCE_VALUE  
    get_resource "mysql" "port"
    local MYSQL_PORT=$G_RESOURCE_VALUE  
    get_resource "mysql" "username"
    local MYSQL_USERNAME=$G_RESOURCE_VALUE  
    get_resource "mysql" "password"
    local MYSQL_PASSWORD=$G_RESOURCE_VALUE  
    local NACOS_LOGS=$G_DATA_DIR/nacos/logs
    local NACOS_PLUGINS=$G_DATA_DIR/nacos/plugins/mysql
    mkdir -p $NACOS_LOGS

    cd $G_INSTALL_DIR/yaml/nacos
    rm -f nacos-install.yaml
    cp nacos.yaml nacos-install.yaml
    sed -i "s+your-mysql-host+$MYSQL_HOST+g" nacos-install.yaml
    sed -i "s+your-mysql-port+$MYSQL_PORT+g" nacos-install.yaml
    sed -i "s+your-mysql-username+$MYSQL_USERNAME+g" nacos-install.yaml
    sed -i "s+your-mysql-password+$MYSQL_PASSWORD+g" nacos-install.yaml
    sed -i "s+your-nacos-logs-path+$NACOS_LOGS+g" nacos-install.yaml    

    kubectl apply -f nacos-install.yaml
}

apply_minio () {
    step "部署 MinIO 中间件"
    local MINIO_DATA=$G_DATA_DIR/minio
    mkdir -p $MINIO_DATA

    get_resource "minio" "username"
    local MINIO_ACCESS_KEY=$G_RESOURCE_VALUE
    get_resource "minio" "password"
    local MINIO_SECRET_KEY=$G_RESOURCE_VALUE
    get_resource "domain" "host"
    local DOMAIN_HOST=$G_RESOURCE_VALUE

    cd $G_INSTALL_DIR/yaml/minio
    rm -f minio-install.yaml
    cp minio.yaml minio-install.yaml
    sed -i "s+your-minio-data-path+$MINIO_DATA+g" minio-install.yaml
    sed -i "s+your-minio-access-key+$MINIO_ACCESS_KEY+g" minio-install.yaml
    sed -i "s+your-minio-secret-key+$MINIO_SECRET_KEY+g" minio-install.yaml
    sed -i "s+your-minio-domain+$DOMAIN_HOST+g" minio-install.yaml

    kubectl apply -f minio-install.yaml
}

apply_elasticsearch () {
    step "部署 ElasticSearch 中间件"
    local ES_DATA=$G_DATA_DIR/elasticsearch/data
    local ES_LOGS=$G_DATA_DIR/elasticsearch/logs
    mkdir -p $ES_DATA
    mkdir -p $ES_LOGS

    chmod -R 777 $G_DATA_DIR/elasticsearch

    cd $G_INSTALL_DIR/yaml/elasticsearch
    rm -f elasticsearch-install.yaml
    cp elasticsearch.yaml elasticsearch-install.yaml
    sed -i "s+your-elasticsearch-data-path+$ES_DATA+g" elasticsearch-install.yaml
    sed -i "s+your-elasticsearch-logs-path+$ES_LOGS+g" elasticsearch-install.yaml

    kubectl apply -f elasticsearch-install.yaml
}

apply_logstash () {
    step "部署 Logstash 中间件"
    cd $G_INSTALL_DIR/yaml/logstash

    get_resource "mysql" "host"
    local MYSQL_HOST=$G_RESOURCE_VALUE  
    get_resource "mysql" "port"
    local MYSQL_PORT=$G_RESOURCE_VALUE  
    get_resource "mysql" "username"
    local MYSQL_USERNAME=$G_RESOURCE_VALUE  
    get_resource "mysql" "password"
    local MYSQL_PASSWORD=$G_RESOURCE_VALUE  
    get_resource "kafka" "host"
    local KAFKA_HOST=$G_RESOURCE_VALUE  
    get_resource "kafka" "port"
    local KAFKA_PORT=$G_RESOURCE_VALUE   
    get_resource "elasticsearch" "host"
    local ELASTICSEARCH_HOST=$G_RESOURCE_VALUE  
    get_resource "elasticsearch" "port"
    local ELASTICSEARCH_PORT=$G_RESOURCE_VALUE

    rm -f logstash-install.yaml
    cp logstash.yaml logstash-install.yaml

    sed -i "s+your-mysql-host+$MYSQL_HOST+g" logstash-install.yaml
    sed -i "s+your-mysql-port+$MYSQL_PORT+g" logstash-install.yaml
    sed -i "s+your-mysql-username+$MYSQL_USERNAME+g" logstash-install.yaml
    sed -i "s+your-mysql-password+$MYSQL_PASSWORD+g" logstash-install.yaml
    sed -i "s+your-kafka-host+$KAFKA_HOST+g" logstash-install.yaml
    sed -i "s+your-kafka-port+$KAFKA_PORT+g" logstash-install.yaml
    sed -i "s+your-elasticsearch-host+$ELASTICSEARCH_HOST+g" logstash-install.yaml
    sed -i "s+your-elasticsearch-port+$ELASTICSEARCH_PORT+g" logstash-install.yaml
    kubectl apply -f logstash-install.yaml
}

apply_kibana () {
    step "部署 Kibana 中间件"
    cd $G_INSTALL_DIR/yaml/kibana

    kubectl apply -f kibana.yaml
}

apply_zookeeper () {
    step "部署 Zookeeper 中间件"
    cd $G_INSTALL_DIR/yaml/kafka
    
    kubectl apply -f zookeeper.yaml
}

apply_kafka () {
    step "部署 Kafka 中间件"
    cd $G_INSTALL_DIR/yaml/kafka
    
    kubectl apply -f kafka.yaml
}

apply_kafka_manager () {
    step "部署 Kafka Manager 中间件"
    cd $G_INSTALL_DIR/yaml/kafka
    
    kubectl apply -f kafka-manager.yaml
}

apply_rocketmq_nameserver () {
    step "部署 RocketMQ NameServer 中间件"
    local NAMESERVER_LOGS=$G_DATA_DIR/rocketmq/logs
    mkdir -p $NAMESERVER_LOGS

    cd $G_INSTALL_DIR/yaml/rocketmq
    rm -f nameserver-install.yaml
    cp nameserver.yaml nameserver-install.yaml
    sed -i "s+your-nameserver-logs-path+$NAMESERVER_LOGS+g" nameserver-install.yaml

    chmod -R 777 $G_DATA_DIR/rocketmq
    kubectl apply -f nameserver-install.yaml
}

apply_rocketmq_broker () {
    step "部署 RocketMQ Broker 中间件"
    local BROKER_LOGS=$G_DATA_DIR/rocketmq/logs
    local BROKER_STORE=$G_DATA_DIR/rocketmq/store
    local BROKER_CONF=$G_DATA_DIR/rocketmq/conf
    mkdir -p $BROKER_LOGS
    mkdir -p $BROKER_STORE
    mkdir -p $BROKER_CONF

    cd $G_INSTALL_DIR/yaml/rocketmq
    rm -f broker-install.yaml
    cp broker.yaml broker-install.yaml
    cp broker.conf $BROKER_CONF/broker.conf

    sed -i "s+your-broker-logs-path+$BROKER_LOGS+g" broker-install.yaml
    sed -i "s+your-broker-store-path+$BROKER_STORE+g" broker-install.yaml
    sed -i "s+your-broker-conf-path+$BROKER_CONF/broker.conf+g" broker-install.yaml

    chmod -R 777 $G_DATA_DIR/rocketmq
    kubectl apply -f broker-install.yaml
}

apply_rocketmq_dashboard () {
    step "部署 RocketMQ Dashboard 中间件"
    local DASHBOARD_ADMIN_PWD="@Lingyun2022"
    local DASHBOARD_LOGS=$G_DATA_DIR/rocketmq/logs
    local DASHBOARD_DATA=$G_DATA_DIR/rocketmq/dashboard
    mkdir -p $DASHBOARD_LOGS
    mkdir -p $DASHBOARD_DATA

    cd $G_INSTALL_DIR/yaml/rocketmq
    rm -f dashboard-install.yaml
    cp dashboard.yaml dashboard-install.yaml
    sed -i "s+your-dashboard-logs-path+$DASHBOARD_LOGS+g" dashboard-install.yaml
    sed -i "s+your-dashboard-data-path+$DASHBOARD_DATA+g" dashboard-install.yaml
    sed -i "s+your-dashboard-admin-password+$DASHBOARD_ADMIN_PWD+g" dashboard-install.yaml

    chmod -R 777 $G_DATA_DIR/rocketmq
    kubectl apply -f dashboard-install.yaml
}

check_nacos () {
    step "推送共享配置等到Nacos"
    local NACOS_HOST="`kubectl get svc -n pai-cloud |grep nacos|awk '{print $3}'|awk -F '/' '{print $1;exit;}'`"

    local NACOS_CHECK_URL=http://$NACOS_HOST:8848/nacos/v1/ns/instance/list?serviceName=nacos 
    local NACOS_CHECK_STATUS=$(curl -s -m 5 -L $NACOS_CHECK_URL) 

    info "\033[32m `date` \033[0m 正等待 Nacos 服务启动, 侦测到服务IP为: $NACOS_HOST" 
    sleep 30s
    while [[ "$NACOS_CHECK_STATUS" == "" ]]; do
        NACOS_CHECK_STATUS=$(curl -s -m 5 -L $NACOS_CHECK_URL)
        sleep 3s
    done

    info "curl -X POST 'http://$NACOS_HOST:8848/nacos/v1/console/namespaces' -d 'customNamespaceId=prod&namespaceName=prod&namespaceDesc=生产环境'"
    local RESULT=`curl -s -X POST "http://$NACOS_HOST:8848/nacos/v1/console/namespaces" -d 'customNamespaceId=prod&namespaceName=prod&namespaceDesc=生产环境'`
    info "create namespace prod : $RESULT"

    # 推送Nacos配置
    if [ -d "$G_INSTALL_DIR/yaml/nacos/config" ]; then
        create_push_nacos        
        find $G_INSTALL_DIR/yaml/nacos/config -maxdepth 1 -name '*.yaml' -type f -exec sh $G_INSTALL_DIR/push_nacos.sh $NACOS_HOST {} \;
        rm -f  $G_INSTALL_DIR/push_nacos.sh
    fi

}

create_push_nacos () {
cat << 'EOF' > $G_INSTALL_DIR/push_nacos.sh
#!/bin/sh

NACOS_HOST=$1
YAML_FILE=$2

urlencode() {
  local string="$1"
  local length="${#string}"
  local encoded=""

  for ((i = 0; i < length; i++)); do
    local char="${string:i:1}"
    case $char in
      ['?&'])
        encoded+="$(printf '%%%02X' "'$char")" ;;
      *)
        encoded+="$char" ;;
    esac
  done

  echo "$encoded"
}

if [[ -f "$YAML_FILE" ]]; then
    CONTENT_DATA=$(cat $YAML_FILE)
    CONTENT_DATA=$(urlencode "$CONTENT_DATA")
    DATA_ID=$(basename $YAML_FILE)
    RESULT=`curl -s -X POST "http://$NACOS_HOST:8848/nacos/v1/cs/configs" -d "tenant=prod&dataId=$DATA_ID&group=DEFAULT_GROUP&type=yaml&content=$CONTENT_DATA"`
    echo "push config $DATA_ID : $RESULT"
fi

EOF

}

apply_datax_web () {
    step "部署 Datax-Web 中间件"
    get_resource "mysql" "host"
    local MYSQL_HOST=$G_RESOURCE_VALUE
    get_resource "mysql" "port"
    local MYSQL_PORT=$G_RESOURCE_VALUE
    get_resource "mysql" "username"
    local MYSQL_USERNAME=$G_RESOURCE_VALUE
    get_resource "mysql" "password"
    local MYSQL_PASSWORD=$G_RESOURCE_VALUE

    cd $G_INSTALL_DIR/yaml/datax-web
    rm -f datax-web-install.yaml
    cp datax-web.yaml datax-web-install.yaml
    sed -i "s+your-mysql-host+$MYSQL_HOST+g" datax-web-install.yaml
    sed -i "s+your-mysql-port+$MYSQL_PORT+g" datax-web-install.yaml
    sed -i "s+your-mysql-username+$MYSQL_USERNAME+g" datax-web-install.yaml
    sed -i "s+your-mysql-password+$MYSQL_PASSWORD+g" datax-web-install.yaml
    kubectl apply -f datax-web-install.yaml
}

apply_pai () {
    step "部署 内核/方案/客户方案 组件"

    local HOST_IP=$(hostname -I | awk '{print $1}')
    get_resource "domain" "host"
    local DOMAIN_HOST=$G_RESOURCE_VALUE
    get_resource "mysql" "host"
    local MYSQL_HOST=$G_RESOURCE_VALUE
    get_resource "mysql" "port"
    local MYSQL_PORT=$G_RESOURCE_VALUE
    get_resource "mysql" "username"
    local MYSQL_USERNAME=$G_RESOURCE_VALUE
    get_resource "mysql" "password"
    local MYSQL_PASSWORD=$G_RESOURCE_VALUE

    for file in "$G_INSTALL_DIR"/yaml/pai/*.yaml; do
      if [ -f "$file" ]; then
        sed -i "s+your-proxy-domain+$DOMAIN_HOST+g" "$file"
        sed -i "s+your-proxy-ip+$HOST_IP+g" "$file"
        sed -i "s+your-mysql-host+$MYSQL_HOST+g" "$file"
        sed -i "s+your-mysql-port+$MYSQL_PORT+g" "$file"
        sed -i "s+your-mysql-username+$MYSQL_USERNAME+g" "$file"
        sed -i "s+your-mysql-password+$MYSQL_PASSWORD+g" "$file"
      fi
    done

    if [[ -f "$G_INSTALL_DIR/yaml/pai/pai-data-sync-edge-agent.yaml" ]]; then
        local META_DATA_PATH=$G_INSTALL_DIR/libs/metadata
        mkdir -p $META_DATA_PATH
        sed -i "s+your-meta-data-path+$META_DATA_PATH+g" $G_INSTALL_DIR/yaml/pai/pai-data-sync-edge-agent.yaml
    fi

    # 先执行先下载，避免同步数据时等太久了
    [ -f $G_INSTALL_DIR/yaml/pai/ConfigMap.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/ConfigMap.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/xxl-job-admin.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/xxl-job-admin.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-config.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-config.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-operate.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-operate.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-appstore.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-appstore.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-open.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-open.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-oss.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-oss.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-auth.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-auth.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-mc.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-mc.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-workflow.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-workflow.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-dp-table.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-dp-table.yaml
    [ -f $G_INSTALL_DIR/yaml/pai/pai-data-sync-edge-agent.yaml ] && kubectl apply -f $G_INSTALL_DIR/yaml/pai/pai-data-sync-edge-agent.yaml

    if [[ -n $(ls $G_INSTALL_DIR/yaml/pai/*.yaml) ]]; then        
        kubectl apply -f $G_INSTALL_DIR/yaml/pai
    fi

    # 2022/08/24 modify lxl 离线包数据太旧，不再下载
    # 当有放 online 文件就表示要用在线实时下载，把已下载的离线包改名
#    if [[ -f "$META_DATA_PATH/online" ]]; then
#        local TIMES="`date +%Y%m%dT%H%M%S`"
#        if [[ -f "$META_DATA_PATH/cei_icon_meta.zip" ]]; then
#            mv "$META_DATA_PATH/cei_icon_meta.zip" "$META_DATA_PATH/cei_icon_meta-$TIMES.zip"
#        fi
#        if [[ -f "$META_DATA_PATH/cei_package_meta.zip" ]]; then
#            mv "$META_DATA_PATH/cei_package_meta.zip" "$META_DATA_PATH/cei_package_meta-$TIMES.zip"
#        fi
#        if [[ -f "$META_DATA_PATH/cei_workflow_meta.zip" ]]; then
#            mv "$META_DATA_PATH/cei_workflow_meta.zip" "$META_DATA_PATH/cei_workflow_meta-$TIMES.zip"
#        fi
#    else
#        info "开始下载离线数据包"
#        if [[ -f "$G_INSTALL_DIR/yaml/pai/pai-data-sync-edge-agent.yaml" ]]; then
#            download_if_not_exist "$META_DATA_PATH/cei_icon_meta.zip" "https://api.flyrise.cn/pai/cei/cei_icon_meta.zip"
#            download_if_not_exist "$META_DATA_PATH/cei_package_meta.zip" "https://api.flyrise.cn/pai/cei/cei_package_meta.zip"
#            download_if_not_exist "$META_DATA_PATH/cei_workflow_meta.zip" "https://api.flyrise.cn/pai/cei/cei_workflow_meta.zip"
#        fi
#        info "离线数据包下载完"
#    fi
}

create_uninstall () {

# 注意这个的EOF前面不能有空格，否则会报错
cat << EOF > $G_DATA_DIR/uninstall.sh
#!/bin/bash
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:~/bin
export PATH
LANG=en_US.UTF-8

G_UNINSTALL_LOG=$G_INSTALL_DIR/uninstall-\`date +%y%m%d%H%M\`.log

uninstall_main () {
    mkdir -p $G_INSTALL_DIR
    read -p "是否导出现有基础镜像（不含业务镜像）到本地, 便于下次安装更有效率 [默认为 N] ? （Y/N）： " IS_BACKUP

    if [[ "\$IS_BACKUP" = [yY] ]]; then
           echo "开始导出镜像，请耐心等待..."
        if command -v docker >/dev/null 2>&1; then
           docker images | grep -v REPOSITORY | grep -v rancher | grep -v "<none>" | \
               awk '{print "name="\$1";name=\${name##*/}-"\$2".tar;echo 导出："\$1":"\$2"\n[ -f $G_INSTALL_DIR/libs/\$name ]||docker save -o $G_INSTALL_DIR/libs/\$name "\$1":"\$2}' | sh
        else 
           crictl images | grep -v IMAGE | grep -v rancher | grep -v "<none>" | \
               awk '{print "name="\$1";name=\${name##*/}-"\$2".tar;echo 导出："\$1":"\$2"\n[ -f $G_INSTALL_DIR/libs/\$name ]||ctr -n k8s.io i export $G_INSTALL_DIR/libs/\$name "\$1":"\$2}' | sh
        fi
    fi

    if [[ -e /usr/local/bin/k3s-uninstall.sh ]]; then
        sh /usr/local/bin/k3s-uninstall.sh
    fi

    if [[ -e /usr/local/bin/k3s-agent-uninstall.sh ]]; then
        sh /usr/local/bin/k3s-agent-uninstall.sh
    fi

    if command -v docker >/dev/null 2>&1; then
        docker rm -f \`docker ps -a -q\`    
        systemctl status docker &&  systemctl stop docker && systemctl disable docker 

        yum remove -y docker docker-common docker-selinux docker-enginece docker-ce docker-ce-cli containerd.io
        #umount \`cat /proc/mounts | grep "docker"| awk '{print \$2}'\` 

        rm -rf $G_DATA_DIR/docker
        rm -rf /var/lib/docker
        rm -rf /var/run/docker
        rm -rf /etc/docker
        rm -rf /usr/libexec/docker
        rm -f /usr/bin/docker 

        # remove containerd
        systemctl disable --now containerd.service

        rm -f /etc/cni/net.d/10-containerd-net.conflist
        rm -f /etc/crictl.yaml
        rm -f /etc/systemd/system/containerd.service
        rm -f /usr/local/bin/containerd-shim-runc-v2
        rm -f /usr/local/bin/ctr
        rm -f /usr/local/bin/containerd-shim
        rm -f /usr/local/bin/containerd-shim-runc-v1
        rm -f /usr/local/bin/crictl
        rm -f /usr/local/bin/critest
        rm -f /usr/local/bin/containerd
        rm -f /usr/local/sbin/runc
        rm -f /opt/cni/bin/vlan
        rm -f /opt/cni/bin/host-local
        rm -f /opt/cni/bin/flannel
        rm -f /opt/cni/bin/bridge
        rm -f /opt/cni/bin/host-device
        rm -f /opt/cni/bin/tuning
        rm -f /opt/cni/bin/firewall
        rm -f /opt/cni/bin/bandwidth
        rm -f /opt/cni/bin/ipvlan
        rm -f /opt/cni/bin/sbr
        rm -f /opt/cni/bin/dhcp
        rm -f /opt/cni/bin/portmap
        rm -f /opt/cni/bin/ptp
        rm -f /opt/cni/bin/static
        rm -f /opt/cni/bin/macvlan
        rm -f /opt/cni/bin/loopback
    fi

    [ -d /opt/containerd ] && rm -rf /opt/containerd
    [ -d /var/lib/containerd ] && rm -rf /var/lib/containerd 
    [ -d /run/containerd ] && rm -rf /run/containerd
    [ -d /var/lib/rancher/k3s ] && rm -rf /var/lib/rancher/k3s
    [ -d /var/lib/kubelet ] && rm -rf /var/lib/kubelet
    [ -d /run/k3s/containerd ] && rm -rf /run/k3s/containerd
    [ -d $G_DATA_DIR/k3s ] && rm -rf $G_DATA_DIR/k3s

    rm -rf /usr/share/kuboard

    [ -f /root/.kube/config ] && rm /root/.kube/config

    #rm -rf $G_DATA_DIR
    find $G_INSTALL_DIR/yaml/ -name *-install.yaml -type f -exec rm -f {} \;

}

uninstall_main 2>&1 | tee \${G_UNINSTALL_LOG}    
EOF
# 注意这个的EOF前面不能有空格，否则会报错

}

install_main () {
    check_network

    local startTimeStr=`date`
    local startTime=`date +%s`
    info "开始安装：\033[32m `date` \033[0m"
    
    system_init
    close_firewalld
    close_swap

    uninstall_k3s
    [ $G_USE_DOCKER -eq 1 ] && uninstall_docker
    create_uninstall
    [ $G_USE_DOCKER -eq 1 ] && install_docker
    install_k3s
    prepare_images

    # G_MASTER_IP 为空表示安装master 否则表示安装agent
    if [ -z ${G_MASTER_IP} ]; then
        apply_kuboard
        change_kuboard_policy
        apply_ingress_controller

        create_namespace
        apply_mysql
        apply_nacos
        apply_redis
        apply_minio

        apply_elasticsearch
        apply_logstash
        apply_kibana

        #apply_zookeeper
        #apply_kafka
        #apply_kafka_manager

        apply_rocketmq_nameserver
        apply_rocketmq_broker
        apply_rocketmq_dashboard
        
        #apply_datax_web
        
        check_nacos

        apply_pai
    fi

    
    local endTimeStr=`date`
    local endTime=`date +%s`
    ((outTime=($endTime-$startTime)/60))
    info "开始时间：\033[32m $startTimeStr \033[0m"
    info "结束时间：\033[32m $endTimeStr \033[0m"
    info "耗费时间：\033[32m $outTime 分钟\033[0m"


    # G_MASTER_IP 为空表示安装master 否则表示安装agent
    if [ "$G_ONLINE" = "1" ] && [ -z ${G_MASTER_IP} ]; then
        info "执行同步脚本"
        sh $G_INSTALL_DIR/cei_sync.sh &
    fi
}

install_main 2>&1 | tee ${G_INSTALL_LOG} 