#!/bin/bash

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:~/bin
export PATH
LANG=en_US.UTF-8

# ==========================================特别说明========================================== #
# 1.以G_开头的变量约定为全局变量(G_XXX)，避免误会局部变量不要以此开头，同时用 local XXX 来声明 #
# 2.Nacos 的默认账码为 nacos @Lingyun2022，配置在yaml/mysql/sqls/nacos-mysql.sql               #
# 3.RocketMQ Dashboard 账码为 admin @Lingyun2022，配置在 apply_rocketmq_dashboard              #
# 4.其他涉及账号密码的请统一查看yaml/resources/resources.yaml                                  #
# ============================================================================================ #

# 获取安装包所在路径
G_INSTALL_DIR=$(cd "$(dirname "$0")";pwd)
G_INSTALL_LOG=$G_INSTALL_DIR/sync-`date +%Y%m%d%H%M`.log

# 默认命名空间
G_NAMESAPCE=${1:-pai-cloud}

# ====================================================

let count=0


# --- helper functions for logs ---
info () {
    echo -e "`date +%Y-%m-%dT%H:%M:%S`" ' [INFO]  ' "$@"
}

warn () {
    echo -e "`date +%Y-%m-%dT%H:%M:%S`" ' [WARN]  ' "$@" >&2
}

fatal () {
    echo -e "`date +%Y-%m-%dT%H:%M:%S`" ' [ERROR] ' "$@" >&2
    info "同步失败，退出程序"
    exit 1
}

#检测网络链接畅通
check_network() {
    #超时时间
    local TIMEOUT=5
    #目标网站
    local TARGET=www.baidu.com
    #获取响应状态码
    local RET_CODE=`curl -I -s --connect-timeout ${TIMEOUT} ${TARGET} -w %{http_code} | tail -n1`
    if [ "x$RET_CODE" = "x200" ]; then
        #网络畅通
        return 0
    fi

    fatal "网络不畅通，请检查网络设置！"   
}

# ====================================================
check_service () {
    local SERVICES=$1
    for NAME in $SERVICES
    do
        info "检查服务：$NAME"
        until RESULT="`kubectl get po -n $G_NAMESAPCE -l app=$NAME --no-headers|awk '{print $2;exit;}'`" && [[ $RESULT =~ '1/' ]]; do 
            echo "等待 $NAME 启动"
            sleep 60s
        done;
        info "检查结果：$RESULT"
    done
}

restart_deployment () {
    NAMESPACE=$1
    DEPLOYMENT=$2

    if kubectl get deploy -n $NAMESPACE | grep -q $DEPLOYMENT; then
      kubectl rollout restart deploy $DEPLOYMENT -n $NAMESPACE
    fi
}

sync_and_restart () {
    check_service "pai-appstore pai-config pai-workflow pai-operate pai-mc pai-oss pai-open pai-dp-table pai-data-sync-edge-agent"
    sleep 30s

    local NAME=pai-data-sync-edge-agent
    local SYNC_HOST="`kubectl get svc $NAME -n $G_NAMESAPCE |grep $NAME |awk '{print $3}'|awk -F '/' '{print $1;exit;}'`"
    
    # 发现第一次请求总是不成功，故意调一下错误的
    curl -sL --retry 0 -X GET http://$SYNC_HOST:8080?k=197010010101 > /dev/nul
    sleep 5s

    for i in {1..3}; do
        local SYNC_URL="http://$SYNC_HOST:8080?force=true&k=`date +%Y%m%d%H%M`"
        info "同步地址：$SYNC_URL"
        local RESULT=`curl -sL --retry 0 -X GET $SYNC_URL`

        info "同步结果：$RESULT"

        if [[ $RESULT =~ '"code":"200"' ]]; then
            restart_deployment $G_NAMESAPCE pai-operate
            restart_deployment $G_NAMESAPCE pai-dp-screen
            restart_deployment $G_NAMESAPCE pai-workflow
            restart_deployment $G_NAMESAPCE pai-wallet-allinpay
            restart_deployment $G_NAMESAPCE pai-dp-data
            
            restart_deployment $G_NAMESAPCE pai-enterprise
            restart_deployment $G_NAMESAPCE pai-business
            restart_deployment $G_NAMESAPCE pai-park-space
            restart_deployment $G_NAMESAPCE pai-finance
            restart_deployment $G_NAMESAPCE pai-park-property
            restart_deployment $G_NAMESAPCE pai-voucher
            restart_deployment $G_NAMESAPCE pai-service-center
            restart_deployment $G_NAMESAPCE pai-work-engine
            restart_deployment $G_NAMESAPCE pai-contract
            restart_deployment $G_NAMESAPCE pai-security
            restart_deployment $G_NAMESAPCE pai-walking
            restart_deployment $G_NAMESAPCE pai-parking
            break
        fi       
        sleep 10s 
    done
}

import_dp_screen () {    
    check_service "pai-dp-screen pai-dp-table"
    
    local NAME=pai-dp-screen
    local SVC_HOST="`kubectl get svc $NAME -n $G_NAMESAPCE |grep -v "\-ui"|grep "$NAME"|awk '{print $3}'|awk -F '/' '{print $1;exit;}'`"
    local IMP_URL=http://$SVC_HOST:8080/data/asset/suite/import
    info "导入上传地址：$IMP_URL"
    
    for i in {1..6}; do        
        local RESULT=`curl --form "file=@$G_INSTALL_DIR/yaml/pai/data_suite.zip" --form 'teamId=39e08787c0a1139b6d9d5b16d5ca3506' --form 'from=shell' $IMP_URL`
        info "第 $i 次导入上传结果：$RESULT"
        if [[ $RESULT =~ '"code":"200"' ]]; then
            info "第 $i 次导入上传成功"
            break
        fi
        info "第 $i 次导入上传失败，30秒后将重试"
        sleep 30s        
    done

    info "最终导入上传结果：$RESULT"
    if [[ $RESULT =~ '"code":"200"' ]]; then
        for i in {1..20}; do
            local IMPORT_RESULT=`curl http://$SVC_HOST:8080/data/asset/suite/import/status?teamId=39e08787c0a1139b6d9d5b16d5ca3506`
            info "第 $i 次导入状态：$IMPORT_RESULT"
            if [[ $IMPORT_RESULT =~ '"status":false' ]]; then
                info "第 $i 次导入状态：结束"
                break
            fi
            info "第 $i 次导入处理中，30秒后重新检查"
            sleep 30s
        done
        restart_deployment $G_NAMESAPCE pai-dp-screen        
    else
        fatal "导入失败！请尝试手动导入：$G_INSTALL_DIR/yaml/pai/data_suite.zip"
    fi
    
}

main () {
    check_network

    local startTimeStr=`date`
    local startTime=`date +%s`
    info "开始同步：\033[32m `date` \033[0m"
    
    sync_and_restart
    sleep 30s
    import_dp_screen
    
    local endTimeStr=`date`
    local endTime=`date +%s`
    ((outTime=($endTime-$startTime)/60))
    info "开始时间：\033[32m $startTimeStr \033[0m"
    info "结束时间：\033[32m $endTimeStr \033[0m"
    info "耗费时间：\033[32m $outTime 分钟\033[0m"
}

main 2>&1 | tee ${G_INSTALL_LOG} 
